# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#
"""
Test sanitisation of --output arg
"""

import os

import pytest

from tartex.tartex import TarTeX


@pytest.fixture
def sample_tex():
    return "sample.tex"


def test_output_default(sample_tex):
    """
    Check correct tar suffix set when output arg ext is not specified
    """
    t = TarTeX([sample_tex, "-o", "main"])
    assert t.tar_file.name == "main.tar"


def test_output_nontar(sample_tex):
    """
    Check correct tar file name is set when output arg does have an extension
    but not one that matches .tar.?z

    Example: `tartex sample.tex -o main.foo.bar`
             should generate "main.foo.bar.tar.gz", not "main.foo.tar.gz"
    """
    out = "main.foo.bar"
    t = TarTeX([sample_tex, "-o", out])
    assert t.tar_file.name == out + ".tar"


def test_output_gz_eqv(sample_tex):
    """
    Check that the three kinds of output file names lead to same final tar file
    """
    common_opts = [sample_tex, "-o"]
    out = "main"
    t1 = TarTeX([*common_opts, out])
    t2 = TarTeX([*common_opts, out + ".gz"])
    t3 = TarTeX([*common_opts, out + ".tar.gz"])

    assert t1.tar_file == t2.tar_file
    assert t2.tar_file == t3.tar_file


def test_tilde_exp(sample_tex):
    """
    Check that '~' specified for output arg expands to user home
    """
    t = TarTeX([sample_tex, "-o", "~/main.tar.xz"])
    assert str(t.tar_file) == os.getenv("HOME") + "/main.tar"



# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#

"""
Tests for bash completion printing and installing
"""

from pathlib import Path

import pytest

from tartex._completion import APPNAME, BashCompletion
from tartex.tartex import TarTeX


def test_print(capsys):
    """Test output of print is not empty"""
    with pytest.raises(SystemExit) as exc:
        TarTeX(["--completion"])

    assert "bash" in capsys.readouterr().out
    assert exc.value.code == 0


def test_install(capsys, monkeypatch, tmpdir):
    """Test installed completions file"""
    monkeypatch.setenv("HOME", str(tmpdir))
    with pytest.raises(SystemExit) as exc:
        TarTeX(["--bash-completion"])

    assert exc.value.code == 0
    bc = BashCompletion()
    compl_file = Path.home() / bc.install_dir / APPNAME
    # For overtly long paths, capsys output may end up introducing "\n" inside
    # the path, make sure we get rid of those when comparing
    assert str(compl_file.parent) in capsys.readouterr().out.replace("\n", "")
    assert compl_file.exists()
    assert compl_file.read_text(encoding="utf-8") == bc.data



# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT

"""Tests for argument parsing"""

import pytest

from tartex.__about__ import __version__
from tartex.tartex import TarTeX, make_tar


class TestArgs:
    """Class to test different combinations of cmdline arguments"""

    def test_no_args(self):
        """Test system exit with missing positional arg"""
        with pytest.raises(SystemExit) as exc:
            make_tar()

        assert exc.value.code == 2

    def test_only_file(self, sample_texfile):
        """Test success with one arg: file name"""
        assert sample_texfile.main_file.stem == "some_file"
        assert sample_texfile.tar_file.name == "some_file.tar"

    def test_version(self, capsys):
        """Test version string against version from __about.py__"""
        # argparse will call SystemExit(0) for -h and -v, and print to stdout
        with pytest.raises(SystemExit) as exc:
            TarTeX(["--version"])

        output = capsys.readouterr().out
        assert f"{__version__}" in output
        assert exc.value.code == 0

    @pytest.mark.parametrize(
        ("tar_opt1", "tar_opt2"), [("-J", "-z"), ("-j", "-J"), ("-z", "-J")]
    )
    def test_taropts_conflict(self, capsys, tar_opt1, tar_opt2):
        """Test exit status when two conflicting tar options are passed"""
        with pytest.raises(SystemExit) as exc:
            TarTeX([tar_opt1, tar_opt2, "some_file.tex"])

        assert exc.value.code == 2
        output = capsys.readouterr().err
        assert "not allowed with" in output



# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#

"""Tests for LaTeX project consisting of multiple dirs"""

import os
import tarfile as tar
from pathlib import Path

import pytest

from tartex.tartex import TAR_DEFAULT_COMP, TarTeX


@pytest.fixture
def multidir_target(datadir):
    return Path(datadir) / "multi"


@pytest.fixture
def multidir_tartex_obj(datadir, multidir_target):
    return TarTeX(
        [
            (Path(datadir) / "main.tex").as_posix(),
            "-v",
            "-s",
            "-o",
            multidir_target.as_posix(),
        ]
    )


@pytest.fixture
def src_files(datadir):
    """Return sorted list of files in source dir"""
    src_files = [
        str(Path(dname).relative_to(datadir) / f)
        for dname, _, files in os.walk(datadir)
        for f in files
    ]

    src_files.sort()  # Sort for comparison with tar output
    return src_files


class TestMultiDir:
    """Tests for LaTeX projects with files spread across multiple dirs"""

    def test_tar(self, src_files, multidir_tartex_obj):
        """Test tar file creation"""
        t = multidir_tartex_obj

        t.tar_files()
        output = t.tar_file.with_suffix(f".tar.{TAR_DEFAULT_COMP}")
        assert output.exists()

        with tar.open(output, "r") as rat:
            # Check if files in tarball have the same dir structure as src_files
            assert src_files == sorted(rat.getnames())

    def test_list(self, datadir):
        """Test printing list of files"""
        t = TarTeX([(Path(datadir) / "main.tex").as_posix(), "-l", "-s"])
        t.tar_files()

        # Tar file must not be created with "-l"
        assert not t.tar_file.with_suffix(".tar.{TAR_DEFAULT_COMP}").exists()



# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#
"""
Test inclusion and exclusion of .ind files
"""

import tarfile as tar

import pytest

from tartex.tartex import TAR_DEFAULT_COMP, TarTeX


@pytest.fixture
def default_args(datadir, tmpdir):
    return [
        str(datadir / "test_index.tex"),
        "-s",
        "-v",
        "-o",
        f"{tmpdir!s}/test_index.tar.{TAR_DEFAULT_COMP}",
    ]


class TestIndex:

    """Test to verify handling of missing .ind file"""

    def test_ind(self, default_args):
        """
        Check: By default tar file must contain '.ind' when it is not excluded
        """
        t = TarTeX(default_args)
        t.tar_files()
        with tar.open(t.tar_file.with_suffix(f".tar.{TAR_DEFAULT_COMP}")) as f:
            # Check test_index.ind file is in tarball even though not in srcdir
            assert "test_index.ind" in f.getnames()
            # Check user/group name attributes
            for attr in ["gname", "uname"]:
                assert not f.getmember("test_index.ind").get_info()[attr]

    def test_ind_excl(self, default_args):
        """
        Check: tar file must not contain '.ind' when it is asked to be excluded
        """
        args = [*default_args, "-x", "*.ind"]
        t = TarTeX(args)
        t.tar_files()
        with tar.open(t.tar_file.with_suffix(f".tar.{TAR_DEFAULT_COMP}")) as f:
            # Check test_index.ind file is not in tarball
            assert "test_index.ind" not in f.getnames()



# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT

"""Tests for different tar compression methods"""

import pytest

from tartex.tartex import TAR_DEFAULT_COMP, TarTeX


@pytest.fixture
def target_tar():
    def _target(tar_ext, cmp_opt=""):
        ttx_opts = ["-v", "-s", "-o", f"dest.tar.{tar_ext}", "some_file.tex"]
        if cmp_opt:
            ttx_opts.append(cmp_opt)
        return TarTeX(ttx_opts)

    return _target


class TestTarExt:
    """
    Class of tests checking automatic tar compression detection based on user
    specified outout tarfile name
    """

    def test_default_compress(self, sample_texfile):
        """Test default tarball extension"""
        assert sample_texfile.tar_ext == TAR_DEFAULT_COMP

    def test_output_file_compress(self, target_tar):
        """Test user output file for different tar.foo extensions"""
        # bz2
        assert target_tar("bz2").tar_ext == "bz2"
        # tar.bz2 overridden by "-J"
        assert target_tar("bz2", "-J").tar_ext != "bz2"
        # xz
        assert target_tar("xz").tar_ext == "xz"
        assert target_tar("xz").tar_file.name == "dest.tar"



# vim:set et sw=4 ts=4:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#
"""
Tests for passing various TeX processors to latexmk
"""

import pytest

from tartex.tartex import TAR_DEFAULT_COMP, TarTeX


@pytest.fixture
def latex_file(datadir):
    return datadir / "latexmk_opts.tex"


@pytest.fixture
def basic_opts(latex_file, tmpdir):
    return [
        str(latex_file),
        "-s",
        "-v",
        "-o",
        str(tmpdir / f"lmk.tar.{TAR_DEFAULT_COMP}"),
    ]


class TestLatexmkOpts:

    """Tests to check various tex processors for latexmk"""

    def test_auto_pdf(self, basic_opts):
        """Check: Automatic processing should use '-pdf'"""
        t = TarTeX(basic_opts)
        t.tar_files()
        assert t.tar_file.with_suffix(f".tar.{TAR_DEFAULT_COMP}").exists()

    def test_ps(self, basic_opts):
        """Check: Processing using -ps"""
        basic_opts.extend(["--latexmk-tex", "ps"])
        t = TarTeX(basic_opts)
        t.tar_files()

    def test_pdflua(self, basic_opts):
        """Check: Processing using -pdflua"""
        basic_opts.extend(["--latexmk-tex", "pdflua"])
        t = TarTeX(basic_opts)
        t.tar_files()
